/**
 *  Copyright 2012-2017 Gunnar Morling (http://www.gunnarmorling.de/)
 *  and/or other contributors as indicated by the @authors tag. See the
 *  copyright.txt file in the distribution for a full listing of all
 *  contributors.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.mapstruct.factory;

import static org.assertj.core.api.Assertions.assertThat;

import org.junit.Test;
import org.mapstruct.test.model.Foo;
import org.mapstruct.test.model.SomeClass;

/**
 * Unit test for {@link Mappers}.
 *
 * @author Gunnar Morling
 */
public class MappersTest {

    @Test
    public void shouldReturnImplementationInstance() {

        Foo mapper = Mappers.getMapper( Foo.class );
        assertThat( mapper ).isNotNull();
    }

    /**
     * Checks if an implementation of a nested mapper can be found. This is a special case since
     * it is named
     */
    @Test
    public void findsNestedMapperImpl() throws Exception {
        assertThat( Mappers.getMapper( SomeClass.Foo.class ) ).isNotNull();
        assertThat( Mappers.getMapper( SomeClass.NestedClass.Foo.class ) ).isNotNull();
    }
}
